@@ -1,27 +1,27 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: awffull.h 475 2007-11-14 06:05:30Z steve $
+    awffull.h
+        The primary definitions file
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
     Copyright 2002, 2004 by Stanislaw Yurievich Pusep
-    Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
+    Copyright (C) 2004-2008 by Stephen McInerney (spm@stedee.id.au)
 
-    This program is free software; you can redistribute it and/or modify
+    This file is part of AWFFull.
+
+    AWFFull is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
 
     This software uses the gd graphics library, which is copyright by
     Quest Protein Database Center, Cold Spring Harbor Labs.  Please
@@ -70,6 +70,8 @@
 
 #define SLOP_VAL 3600                           /* out of sequence slop (seconds)   */
 
+#define VOLUMEBASE          1000                /* Move to SI Units. Kb = 1000 vs 1024.
+                                                   If change to 1024, modify hr_size() in output.c */
 
 /* Default Graph Sizes */
 #define GRAPH_INDEX_X 512                       /* px. Default X size (512)             */
@@ -194,9 +196,6 @@
 #define VPRINT(level, ...) if (g_settings.settings.verbosity >= (level)) { printf(__VA_ARGS__); }
 #define ERRVPRINT(level, ...) fflush (stdout); if (g_settings.settings.verbosity >= (level)) { fprintf(stderr, __VA_ARGS__); }
 
-//#define PCENT(val,max) ((val)?((double)(val)/(double)(max))*100.0 : 0.0)
-//#define INVPCENT(val,max) ((val)?(1.0 - ((double)(val)/(double)(max)))*100.0 : 0.0)
-
 #define IDX_2C(c1,c2)       ((((c1) -'a'+1)<<5)+((c2) -'a'+1) )
 #define IDX_3C(c1,c2,c3)    ((((c1) -'a'+1)<<10)+(((c2) -'a'+1)<<5)+((c3) -'a'+1) )
 #define IDX_4C(c1,c2,c3,c4) ((((c1) -'a'+1)<<15)+(((c2) -'a'+1)<<10)+(((c3) -'a'+1)<<5)+((c4) -'a'+1) )
@@ -310,6 +309,8 @@ struct generic_flags {
     bool shade_groups;                          /* Group shading 0=no 1=yes                                    */
     bool highlight_groups;                      /* Group hlite 0=no 1=yes                                      */
     bool incremental;                           /* incremental mode 1=yes                                      */
+    bool incremental_duplicate_check;           /* Set to true when in incremental mode;
+                                                   false when not in incremental mode, or already found all duplicates */
     bool use_https;                             /* use 'https://' on URL's                                     */
     bool fold_seq_err;                          /* fold seq err (0=no)                                         */
     bool hide_sites;                            /* Hide ind. sites (0=no)                                      */
@@ -320,6 +321,12 @@ struct generic_flags {
     bool use_geoip;                             /* If able, use GeoIP capability                               */
     bool have_geoip;                            /* If GeoIP is compiled in                                     */
     bool cumulative_percentages;                /* Display Cumulative percentages instead of by share          */
+    bool segmenting;                            /* True if we are doing any Segmenting                         */
+    bool segcountry;                            /* True if we are Segmenting by Country                        */
+    bool segreferer;                            /* True if we are Segmenting by Referer                        */
+    bool disable_report_file_checks;            /* True if we are NOT doing any checks for CSS files and links to old reports */
+    bool ignore_index_alias;                    /* True if we ignore index.* checks in the URL                 */
+    bool is_first_run;
 };
 
 /* General Settings, not otherwise categorised */
@@ -340,6 +347,7 @@ struct generic_settings {
     const char *out_dir;                        /* output directory                                                     */
     const char *flags_location;                 /* Display Country Flags if not null                                    */
     const char *geoip_database;                 /* location of the GeoIP Database file                                  */
+    unsigned history_index;                     /* Current place in the history - more of a counter, but counters are reset monthly */
 };
 
 /* The flags for displaying ALL_XXX results */
@@ -407,9 +415,9 @@ extern struct global_settings g_settings;
  * All that can be counted....
  *************************************************************************/
 struct generic_counters {
-    unsigned long bad;                          /* total bad records        */
-    unsigned long error;                        /* total error pages        */
-    unsigned long ignored;                      /* total ignored records    */
+    unsigned long bad_month;                    /* total bad records        */
+    unsigned long error_month;                  /* total error pages        */
+    unsigned long ignored_month;                /* total ignored records    */
 };
 
 struct hourly_counters {
@@ -443,6 +451,8 @@ struct daily_counters {
 };
 
 struct monthly_counters {
+    unsigned int first_day;
+    unsigned int last_day;
     unsigned long hit;
     unsigned long partial_hit;
     unsigned long file;
@@ -468,8 +478,16 @@ struct global_counters {
     struct monthly_counters month;
 };
 
+struct global_run_counters {
+    unsigned long total_run;                    /* Total Records Processed     */
+    unsigned long bad_run;                      /* total bad records        */
+    unsigned long error_run;                    /* total error pages        */
+    unsigned long ignored_run;                  /* total ignored records    */
 
-extern struct global_counters g_counters;       /* All counters             */
+};
+
+extern struct global_counters g_counters;       /* All monthly/daily/hourly counters         */
+extern struct global_run_counters g_run_counters;       /* All permanent for the entire run counters */
 
 
 extern char *log_fname;                         /* log file pointer         */
@@ -481,7 +499,6 @@ extern GeoIP *gi;                               /* GeoIP access             */
 
 extern unsigned long cur_tstamp;                /* Current timestamp        */
 extern unsigned long epoch;                     /* used for timestamp adj.  */
-extern int check_dup;                           /* check for dups flag      */
 
 extern const char *version;                     /* program version          */
 extern const char *copyright;
@@ -495,7 +512,6 @@ extern int g_cur_year, g_cur_month,             /* year/month/day/hour      */
 extern unsigned long dt_site;                   /* daily 'sites' total      */
 extern unsigned long ht_hit, mh_hit;            /* hourly hits totals       */
 
-extern int f_day, l_day;                        /* first/last day vars      */
 extern int gz_log;                              /* flag for zipped log      */
 
 extern CLISTPTR *top_ctrys;                     /* Top countries table      */
@@ -506,7 +522,6 @@ extern char site_color[];                       /* graph site color         */
 extern char kbyte_color[];                      /* graph kbyte color        */
 extern char page_color[];                       /* graph page color         */
 extern char visit_color[];                      /* graph visit color        */
-extern char bookm_color[];                      /* graph bookm color        */
 extern char pie_color1[];                       /* pie additionnal color 1  */
 extern char pie_color2[];                       /* pie additionnal color 2  */
 extern char pie_color3[];                       /* pie additionnal color 3  */
@@ -522,6 +537,8 @@ extern void get_config(const char *);           /* Read a config file       */
 extern void generate_css_file(void);            /* Generate the CSS file    */
 extern void set_defaults(void);                 /* Use the default-global structures to set the default settings */
 extern char *get_domain(char *);                /* return domain name  */
+extern char *strtoupper(char *);                /* Convert a string to Upper Case   */
+extern char *strtolower(char *);                /* Convert a string to Lower Case   */
 
 extern unsigned long jdate(int, int, int);
 extern void assign_messages(void);
@@ -534,6 +551,7 @@ extern void assign_messages(void);
 #include "parser.h"
 #include "preserve.h"
 #include "xmalloc.h"
+#include "segment.h"
 
 
 #endif          /* AWFFULL_AWFFULL_H */
