@@ -1,33 +1,34 @@
 /*
-    graphs.c  - produces graphs used by AWFFull
-    
-    $Id: graphs.c 475 2007-11-14 06:05:30Z steve $
+    AWFFull - A Webalizer Fork, Full o' features
 
+    graphs.c
+        produces graphs used by AWFFull
+    
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
-	Copyright (C) 2004,2005, 2006, 2007  Stephen P. McInerney (spm@stedee.id.au)
+	Copyright (C) 2004-2008  Stephen McInerney (spm@stedee.id.au)
     Copyright (C) 2006 by Benoit Rouits (brouits@free.fr)
 	
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    This file is part of AWFFull.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
 
     This software uses the gd graphics library, which is copyright by
     Quest Protein Database Center, Cold Spring Harbor Labs.  Please
     see the documentation supplied with the library for additional
     information and license terms, or visit www.boutell.com/gd/ for the
     most recent version of the library and supporting documentation.
+
 */
 
 #include <float.h>
@@ -232,7 +233,7 @@ draw_horiz_grid(unsigned long long max,         /* the max value
                 int height,                     /* px. Height of the "frame"            */
                 int text_offset,                /* px. Offset for grid numbering
                                                    Can be -'ve                          */
-                int is_kbytes                   /* Boolean. Set to 1 if unit is Kbytes - 1024 */
+                int is_kbytes                   /* Boolean. Set to 1 if unit is Kbytes - VOLUMEBASE */
     )
 {
     double grid;                                /* the grid spacing to use              */
@@ -251,6 +252,8 @@ draw_horiz_grid(unsigned long long max,         /* the max value
 
     double grid_cnt = 0.0;                      /* How many grid lines displayed. Used to avoid double probs    */
     int bounding_rectange[8];                   /* GD Bounding Rectangle, use to identify outer limits of drawn text */
+    char *imagestr_rtn = NULL;                  /* Showing the return value from gdImageStringFT() - assists with debugging font woes */
+    static bool imagestr_errored = false;       /* Only display the gdImageStringFT error once. Repeats of the same thing don't help */
 
     VPRINT(VERBOSE4, "Draw Horiz Grid: Max: %llu\n", max);
 
@@ -261,7 +264,7 @@ draw_horiz_grid(unsigned long long max,         /* the max value
             j /= 1000.0;
             pref_idx++;
         }
-        max = max / ((pow(1024.0, (double) pref_idx))) * pow(1000.0, (double) pref_idx);
+        max = max / ((pow((double) VOLUMEBASE, (double) pref_idx))) * pow(1000.0, (double) pref_idx);
         max_f = (double) max;
         VPRINT(VERBOSE5, "    Vol Horiz Grid: Max: %llu, Index: %d\n", max, pref_idx);
     }
@@ -303,7 +306,11 @@ draw_horiz_grid(unsigned long long max,         /* the max value
         gdImageLine(im, xleft, ytop, xright, ytop, dkgrey);
 #ifdef HAVE_GDIMAGESTRINGFT
         /* "print" the string to get an idea of actual size and hence find the centre */
-        gdImageStringFT(NULL, &bounding_rectange[0], 0, graph_font_default, GRAPH_FONT_SIZE_TINY, 0.0, 0, 0, str);
+        imagestr_rtn = gdImageStringFT(NULL, &bounding_rectange[0], 0, graph_font_default, GRAPH_FONT_SIZE_TINY, 0.0, 0, 0, str);
+        if (imagestr_rtn && imagestr_errored != true) {
+            ERRVPRINT(VERBOSE1, "Error: Invalid gdImageStringFT: %s --> %s  Font: %s\n", str, imagestr_rtn, graph_font_default);
+            imagestr_errored = true;
+        }
         text_centring = (int) (((double) bounding_rectange[4] / 2.0) + 1.0);
 
         /* Final check to verify that we're not over-writing existing text */
@@ -807,7 +814,7 @@ year_graph6x(char *fname,                       /* file name use
     }
     if (g_settings.graphs.lines) {
         edge_text_offset = section_left - section_right - GRAPH_TEXT_Y_OFFSET;
-        draw_horiz_grid((unsigned long long) fmaxval * 1024, section_left + 1, section_right, section_bottom, section_bottom - section_middle - GRAPH_TB_IN_OFFSET - 1,
+        draw_horiz_grid((unsigned long long) fmaxval * VOLUMEBASE, section_left + 1, section_right, section_bottom, section_bottom - section_middle - GRAPH_TB_IN_OFFSET - 1,
                         edge_text_offset, 1);
     }
     bar_width = calc_bar_width(section_right - section_left, index_diff, &bar_gap, &bar_sep, 1);
