@@ -1,25 +1,26 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
+    messages.c
+        Setup for common messages to the cmd line
 
-    $Id: messages.c 469 2007-11-07 09:54:13Z steve $
+    Copyright (C) 2004-2008 by Stephen McInerney (spm@stedee.id.au)
 
-    This program is free software; you can redistribute it and/or modify
+    This file is part of AWFFull.
+
+    AWFFull is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
+
 */
 
 /***************************************************************************
@@ -42,6 +43,31 @@ struct country_code ctry[300];
 struct response_code response[TOTAL_RC];
 
 
+/*
+  month name, short or long
+  returns newly allocated string
+*/
+static char *
+get_month_name(unsigned int month, bool short_name)
+{
+    char date[32];
+    struct tm t;
+    size_t name_size = 100;
+    size_t ret;
+    char *name = XMALLOC(char, name_size);
+
+    snprintf(date, sizeof(date), "2001-%02u-12 18:31:01", month);
+    strptime(date, "%Y-%m-%d %H:%M:%S", &t);
+    ret = strftime(name, name_size, (short_name ? "%b" : "%B"), &t);
+    if (ret == 0 || ret > name_size) {
+        ERRVPRINT(VERBOSE0, "FATAL ERROR! get_month_name() failed: %d %d %d\n", month, short_name, ret);
+        exit(1);
+    }
+
+    return name;
+}
+
+
 /***********************************************************************/
 
 void
@@ -49,6 +75,7 @@ assign_messages(void)
 {
     unsigned int i = 0;                         /* Basic counters for use with INIT_CTRY macro */
     unsigned int j = 0;
+    unsigned month;
 
     h_msg = _("\
   -h, --help                 Print this help message\n\
@@ -92,41 +119,23 @@ assign_messages(void)
   -e --top_entry=num         Display num top Entry Pages\n\
   -E --top_exit=num          Display num top Exit Pages\n\
   -g num                     Group Domains to 'num' levels\n\
+\n\
+  --seg_country=COUNTRY      Segment based on the Country\n\
+  --seg_referer=REFERER      Segment based on the Referal Text\n\
 \n\
   -X                         Hide individual sites\n\
   -M num                     Mangle Agent Level (0-6; 0=disable)\n\
 \n\
   --use_geoip                Enable GeoIP lookups\n\
   --match_counts             Display the various Group/Hide etc Match Counts\n\
+  --disable_file_checks      Disable File Sytem Checks for Monthly Reports\n\
 ");
 
-/* short month names MUST BE 3 CHARS in size... pad if needed*/
-    s_month[0] = _("Jan");
-    s_month[1] = _("Feb");
-    s_month[2] = _("Mar");
-    s_month[3] = _("Apr");
-    s_month[4] = _("May");
-    s_month[5] = _("Jun");
-    s_month[6] = _("Jul");
-    s_month[7] = _("Aug");
-    s_month[8] = _("Sep");
-    s_month[9] = _("Oct");
-    s_month[10] = _("Nov");
-    s_month[11] = _("Dec");
 
-/* long month names - can be any length */
-    l_month[0] = _("January");
-    l_month[1] = _("February");
-    l_month[2] = _("March");
-    l_month[3] = _("April");
-    l_month[4] = _("May");
-    l_month[5] = _("June");
-    l_month[6] = _("July");
-    l_month[7] = _("August");
-    l_month[8] = _("September");
-    l_month[9] = _("October");
-    l_month[10] = _("November");
-    l_month[11] = _("December");
+    for (month = 1; month <= 12; ++month) {
+        s_month[month - 1] = get_month_name(month, true);
+        l_month[month - 1] = get_month_name(month, false);
+    }
 
 /* Init the respone code array */
     memset(response, 0, sizeof(response));
