@@ -1,34 +1,28 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
+    options.c
+        Process command line options
+        set default flag/option values
 
-    $Id: options.c 475 2007-11-14 06:05:30Z steve $
+    Copyright (C) 2004-2008 by Stephen McInerney (spm@stedee.id.au)
 
-    This program is free software; you can redistribute it and/or modify
+    This file is part of AWFFull.
+
+    AWFFull is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
-*/
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
 
-/***************************************************************************
- ***************************************************************************
- * options.c
- *
- * Deal with Command line options
- ***************************************************************************
- ***************************************************************************/
+*/
 
 #include "awffull.h"                            /* main header              */
 
@@ -57,8 +51,8 @@ display_version(void)
     printf("%s\n", PACKAGE_STRING);
     printf(_("\
 Copyright 1997-2001 by Bradford L. Barrett\n\
-Copyright 2002, 2004 by Stanislaw Yurievich Pusep\n\
-Copyright 2004, 2005, 2006, 2007 by Steve McInerney\n\n"));
+Copyright 2004-2008 by Steve McInerney\n\
+Other copyrights, see the source files\n"));
 
 }
 
@@ -87,10 +81,9 @@ display_gnu_info(int exit_code)
 {
     display_version();
     printf(_("\
-There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
-PARTICULAR PURPOSE.  You may redistribute copies of %s under the terms\n\
-of the GNU General Public License.\n\
-For more information about these matters, see the file named COPYING.\n"), PACKAGE_NAME);
+Licence GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+This is free software: you are free to change and redistribute it.\n\
+There is NO WARRANTY, to the extent permitted by law.\n"));
     exit(exit_code);
 }
 
@@ -110,6 +103,7 @@ process_options(int argc, char *argv[])
     const struct option long_options[] = {
         {"config", 1, NULL, 'c'},
         {"debug", 0, NULL, 'd'},
+        {"disable_file_checks", 0, NULL, 0},
         {"fold", 0, NULL, 'f'},
         {"help", 0, NULL, 'h'},
         {"ignore_history", 0, NULL, 'i'},
@@ -130,6 +124,9 @@ process_options(int argc, char *argv[])
         {"verbose", 0, NULL, 'v'},
         {"version", 0, NULL, 'V'},
         {"vquiet", 0, NULL, 'Q'},
+        {"seg_country", 1, NULL, 0},
+        {"seg_referer", 1, NULL, 0},
+        {"seg_search", 1, NULL, 0},
         {NULL, 0, NULL, 0}
     };
 
@@ -141,6 +138,20 @@ process_options(int argc, char *argv[])
                 g_settings.flags.display_match_counts = true;
             } else if (strncmp("use_geoip", long_options[option_index].name, strlen("use_geoip")) == 0) {
                 g_settings.flags.use_geoip = true;
+            } else if (strncmp("seg_country", long_options[option_index].name, strlen("seg_country")) == 0) {
+                /* TODO: Move the vprint to a listdump at the end of all options loaded */
+                add_list_member(strtoupper(optarg), &seg_countries, IGNORESPACE);
+                g_settings.flags.segcountry = true;
+                VPRINT(VERBOSE1, "Segmenting by Country: %s\n", optarg);
+                g_settings.flags.segmenting = true;
+            } else if (strncmp("seg_referer", long_options[option_index].name, strlen("seg_referer")) == 0) {
+                add_list_member(optarg, &seg_referers, IGNORESPACE);
+                g_settings.flags.segreferer = true;
+                VPRINT(VERBOSE1, "Segmenting by Referer: %s\n", optarg);
+                g_settings.flags.segmenting = true;
+            } else if (strncmp("disable_file_checks", long_options[option_index].name, strlen("disable_file_checks")) == 0) {
+                VPRINT(VERBOSE1, "Disabling Checks for Existing CSS and Monthly Reports\n");
+                g_settings.flags.disable_report_file_checks = true;
             }
             break;
         case 'a':
@@ -338,6 +349,7 @@ set_defaults(void)
     g_settings.flags.shade_groups = true;
     g_settings.flags.highlight_groups = true;
     g_settings.flags.incremental = false;
+    g_settings.flags.incremental_duplicate_check = false;
     g_settings.flags.use_https = false;
     g_settings.flags.fold_seq_err = false;
     g_settings.flags.hide_sites = false;
@@ -348,6 +360,12 @@ set_defaults(void)
     g_settings.flags.use_geoip = false;
     g_settings.flags.have_geoip = false;
     g_settings.flags.cumulative_percentages = true;
+    g_settings.flags.segmenting = false;
+    g_settings.flags.segcountry = false;
+    g_settings.flags.segreferer = false;
+    g_settings.flags.disable_report_file_checks = false;
+    g_settings.flags.ignore_index_alias = false;
+    g_settings.flags.is_first_run = true;
 
     g_settings.all.sites = false;
     g_settings.all.urls = false;
@@ -372,27 +390,28 @@ set_defaults(void)
     g_settings.dump.dump_path = NULL;
     g_settings.dump.dump_ext = "tab";
 
+    g_settings.settings.index_months = MAXHISTLEN;
     g_settings.settings.mangle_agent = 0;
     g_settings.settings.visit_timeout = 1800;
     g_settings.settings.log_type = LOG_AUTO;
     g_settings.settings.group_domains = 0;
+    g_settings.settings.verbosity = VERBOSE0;
     g_settings.settings.hostname = NULL;
     // The default title to display on the HTML Reports
     g_settings.settings.title_message = _("Usage Statistics for");
     g_settings.settings.state_filename = FILENAME_CURRENT;
     g_settings.settings.history_filename = FILENAME_HISTORY;
-    g_settings.settings.html_ext = "html";
     g_settings.settings.css_filename = FILENAME_CSS;
 
     g_settings.settings.config_filename = XMALLOC(char, strlen(ETCDIR) + strlen(FILENAME_CONFIG) + 2);
 
     snprintf(g_settings.settings.config_filename, strlen(ETCDIR) + strlen(FILENAME_CONFIG) + 2, "%s/%s", ETCDIR, FILENAME_CONFIG);
 
+    g_settings.settings.html_ext = "html";
     g_settings.settings.out_dir = NULL;
-    g_settings.settings.index_months = 12;
     g_settings.settings.flags_location = NULL;
     g_settings.settings.geoip_database = GEOIP_DATABASE;
-    g_settings.settings.verbosity = VERBOSE0;
+    g_settings.settings.history_index = 0;
 
     g_settings.top.sites = 30;
     g_settings.top.sites_by_vol = 10;
@@ -462,6 +481,12 @@ display_options(void)
         VPRINT(0, "g_settings.flags.use_geoip: %u\n", g_settings.flags.use_geoip);
         VPRINT(0, "g_settings.flags.have_geoip: %u\n", g_settings.flags.have_geoip);
         VPRINT(0, "g_settings.flags.cumulative_percentages: %u\n", g_settings.flags.cumulative_percentages);
+        VPRINT(0, "g_settings.flags.segmenting: %u\n", g_settings.flags.segmenting);
+        VPRINT(0, "g_settings.flags.segcountry: %u\n", g_settings.flags.segcountry);
+        VPRINT(0, "g_settings.flags.segreferer: %u\n", g_settings.flags.segreferer);
+        VPRINT(0, "g_settings.flags.disable_stat_checks: %u\n", g_settings.flags.disable_report_file_checks);
+        VPRINT(0, "g_settings.flags.ignore_index_alias: %u\n", g_settings.flags.ignore_index_alias);
+        VPRINT(0, "g_settings.flags.is_first_run: %u\n", g_settings.flags.is_first_run);
 
         VPRINT(0, "g_settings.all.sites: %u\n", g_settings.all.sites);
         VPRINT(0, "g_settings.all.urls: %u\n", g_settings.all.urls);
