@@ -1,44 +1,36 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: parser.c 440 2007-08-05 12:20:41Z steve $
+    parser.c
+        parsing log lines and individual records
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
-    Copyright (C) 2004, 2005, 2006 by Stephen McInerney (spm@stedee.id.au)
+    Copyright (C) 2004-2008 by Stephen McInerney (spm@stedee.id.au)
     Copyright (C) 2006 by John Heaton (john@manchester.ac.uk)
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    This file is part of AWFFull.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
 
     This software uses the gd graphics library, which is copyright by
     Quest Protein Database Center, Cold Spring Harbor Labs.  Please
     see the documentation supplied with the library for additional
     information and license terms, or visit www.boutell.com/gd/ for the
     most recent version of the library and supporting documentation.
+
 */
 
-
-/* Parser Module
- * All code related to parsing log lines and individual records, move here
- */
-
-/*********************************************/
-/* STANDARD INCLUDES                         */
-/*********************************************/
-
 #include "awffull.h"                            /* main header              */
 
 /* internal function prototypes */
@@ -587,8 +579,11 @@ identify_log_format(char *buffer)
 
     rc = pcre_exec(cmp_log_regexp_xferlog, NULL, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
     if (rc >= 0) {
-        /* Matches against COMBINED */
+        /* Matches against FTP/XFERLOG */
         VPRINT(VERBOSE1, "%s\n", _("Using FTP/XFERLOG Log Format"));
+        /* Invalid tables for this log type. Zero them away and hence not display. */
+        g_settings.top.agents = 0;
+        g_settings.top.refs = 0;
         return (LOG_FTP);
     }
 
@@ -596,6 +591,9 @@ identify_log_format(char *buffer)
     if (rc >= 0) {
         /* Matches against SQUID */
         VPRINT(VERBOSE1, "%s\n", _("Using SQUID Log Format"));
+        /* Invalid tables for this log type. Zero them away and hence not display. */
+        g_settings.top.agents = 0;
+        g_settings.top.refs = 0;
         return (LOG_SQUID);
     }
 
