@@ -1,38 +1,35 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: parser.h 431 2007-07-20 10:49:06Z steve $ 
+    parser.h
+        log file parsing definitions/includes
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
-    Copyright (C) 2004, 2005, 2006 by Stephen McInerney (spm@stedee.id.au)
+    Copyright (C) 2004, 2005, 2006, 2008 by Stephen McInerney (spm@stedee.id.au)
     Copyright (C) 2006 by John Heaton (john@manchester.ac.uk)
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    This file is part of AWFFull.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
 
     This software uses the gd graphics library, which is copyright by
     Quest Protein Database Center, Cold Spring Harbor Labs.  Please
     see the documentation supplied with the library for additional
     information and license terms, or visit www.boutell.com/gd/ for the
     most recent version of the library and supporting documentation.
-*/
 
-/* Parser Module
- * All code related to parsing log lines and individual records, move here
- */
+*/
 
 #ifndef AWFFULL_PARSER_H
 #define AWFFULL_PARSER_H
