@@ -1,33 +1,34 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: preserve.c 438 2007-07-31 10:14:43Z steve $
+    preserve.c
+        preserve 'state' between runs
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
-    Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
+    Copyright (C) 2004-2008 by Stephen McInerney (spm@stedee.id.au)
     Copyright (C) 2006 by Alexander Lazic (al-awffull@none.at)
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version, and provided that the above
-    copyright and permission notice is included with all distributed
-    copies of this or derived software.
+    This file is part of AWFFull.
 
-    This program is distributed in the hope that it will be useful,
+    AWFFull is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    AWFFull is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+    along with AWFFull.  If not, see <http://www.gnu.org/licenses/>.
 
     This software uses the gd graphics library, which is copyright by
     Quest Protein Database Center, Cold Spring Harbor Labs.  Please
     see the documentation supplied with the library for additional
     information and license terms, or visit www.boutell.com/gd/ for the
     most recent version of the library and supporting documentation.
+
 */
 
 #include "awffull.h"                            /* main header              */
@@ -98,8 +99,8 @@ get_history()
         history_list[i].xfer = 0.0;
         history_list[i].page = 0;
         history_list[i].visit = 0;
-        history_list[i].fday = 0;
-        history_list[i].lday = 0;
+        history_list[i].first_day = 0;
+        history_list[i].last_day = 0;
     }
 
     /* Open History file and Process */
@@ -114,7 +115,7 @@ get_history()
                                &history_list[nbr_hist_entries].hit,
                                &history_list[nbr_hist_entries].file,
                                &history_list[nbr_hist_entries].site,
-                               &history_list[nbr_hist_entries].xfer, &history_list[nbr_hist_entries].fday, &history_list[nbr_hist_entries].lday,
+                               &history_list[nbr_hist_entries].xfer, &history_list[nbr_hist_entries].first_day, &history_list[nbr_hist_entries].last_day,
                                &history_list[nbr_hist_entries].page, &history_list[nbr_hist_entries].visit);
             if (numfields == 8) {               /* kludge for reading 1.20.xx history files */
                 history_list[nbr_hist_entries].page = 0;
@@ -130,6 +131,11 @@ get_history()
         fclose(hist_fp);
         /* Sort the history array. We only need to sort those entries we've just received */
         qsort(history_list, nbr_hist_entries, sizeof(struct history), cmp_history);
+        if (nbr_hist_entries > 0) {
+            g_counters.month.first_day = history_list[nbr_hist_entries - 1].first_day;
+            g_counters.month.last_day = history_list[nbr_hist_entries - 1].last_day;
+        }
+        VPRINT(VERBOSE2, "History Read. %d Entries\n", nbr_hist_entries);
     } else {
         VPRINT(VERBOSE1, "%s\n", _("History file not found..."));
     }
@@ -149,12 +155,10 @@ put_history()
 
     if (hist_fp) {
         VPRINT(VERBOSE1, "%s\n", _("Saving history information..."));
-        for (i = 0; i < MAXHISTLEN; i++) {
-            if ((history_list[i].month != 0) && (history_list[i].hit != 0)) {
-                fprintf(hist_fp, "%d %d %lu %lu %lu %.0f %d %d %lu %lu\n",
-                        history_list[i].month, history_list[i].year, history_list[i].hit, history_list[i].file, history_list[i].site, history_list[i].xfer, history_list[i].fday,
-                        history_list[i].lday, history_list[i].page, history_list[i].visit);
-            }
+        for (i = 0; i < MAXHISTLEN && i < (g_settings.settings.history_index + 1); i++) {
+            fprintf(hist_fp, "%d %d %lu %lu %lu %.0f %d %d %lu %lu\n",
+                    history_list[i].month, history_list[i].year, history_list[i].hit, history_list[i].file, history_list[i].site, history_list[i].xfer, history_list[i].first_day,
+                    history_list[i].last_day, history_list[i].page, history_list[i].visit);
         }
         fclose(hist_fp);
     } else {
@@ -162,6 +166,112 @@ put_history()
     }
 }
 
+
+/************************************************************************
+ * update_history_array                                                 *
+ *                                                                      *
+ * Updates the history Structure Array.                                 *
+ * Should be run at the end of every month processed.                   *
+ *                                                                      *
+ ************************************************************************/
+void
+update_history_array(void)
+{
+    unsigned int year_index, month_index, history_index;
+    unsigned int i, j;
+
+    VPRINT(VERBOSE2, "Updating History \"by month\" structure\n");
+    g_settings.settings.history_index = 0;
+    if (history_list[0].year == 0) {
+        VPRINT(VERBOSE2, "History Init!\n");
+        /* Empty History; Create 1st entry. */
+        history_list[0].year = g_cur_year;
+        history_list[0].month = g_cur_month;
+        history_list[0].hit = g_counters.month.hit;
+        history_list[0].file = g_counters.month.file;
+        history_list[0].site = g_counters.month.site;
+        history_list[0].xfer = g_counters.month.vol / VOLUMEBASE;
+        history_list[0].first_day = g_counters.month.first_day;
+        history_list[0].last_day = g_counters.month.last_day;
+        history_list[0].page = g_counters.month.page;
+        history_list[0].visit = g_counters.month.visit;
+    } else {
+        history_index = ((g_cur_year - history_list[0].year) * 12) + g_cur_month - history_list[0].month;
+        if (history_index >= MAXHISTLEN) {
+            /* We've managed to exceed the size of the history list!
+             * shuffle everything down */
+            VPRINT(VERBOSE2, "History Shuffle! Max: %u --> %u\n", MAXHISTLEN, history_index);
+            j = history_index - MAXHISTLEN + 1;
+            for (i = 0; i < MAXHISTLEN; i++) {
+//                VPRINT(0, "Index %u - %u  From: %d/%d ", i,j,history_list[i].year, history_list[i].month);
+                if ((i + j) >= MAXHISTLEN) {
+//                    VPRINT(0, " to %d/%d  ZERO\n", 0, 0);
+                    history_list[i].year = 0;
+                    history_list[i].month = 0;
+                    history_list[i].hit = 0;
+                    history_list[i].file = 0;
+                    history_list[i].site = 0;
+                    history_list[i].xfer = 0;
+                    history_list[i].first_day = 0;
+                    history_list[i].last_day = 0;
+                    history_list[i].page = 0;
+                    history_list[i].visit = 0;
+                } else {
+//                    VPRINT(0, " to %d/%d  COPY\n", history_list[i + j].year, history_list[i + j].month);
+                    history_list[i].year = history_list[i + j].year;
+                    history_list[i].month = history_list[i + j].month;
+                    history_list[i].hit = history_list[i + j].hit;
+                    history_list[i].file = history_list[i + j].file;
+                    history_list[i].site = history_list[i + j].site;
+                    history_list[i].xfer = history_list[i + j].xfer;
+                    history_list[i].first_day = history_list[i + j].first_day;
+                    history_list[i].last_day = history_list[i + j].last_day;
+                    history_list[i].page = history_list[i + j].page;
+                    history_list[i].visit = history_list[i + j].visit;
+                }
+            }
+            history_index = MAXHISTLEN - 1;
+        }
+//        VPRINT(0, " DONE!\n");
+        history_list[history_index].year = g_cur_year;
+        history_list[history_index].month = g_cur_month;
+        history_list[history_index].hit = g_counters.month.hit;
+        history_list[history_index].file = g_counters.month.file;
+        history_list[history_index].site = g_counters.month.site;
+        history_list[history_index].xfer = g_counters.month.vol / VOLUMEBASE;
+        history_list[history_index].first_day = g_counters.month.first_day;
+        history_list[history_index].last_day = g_counters.month.last_day;
+        history_list[history_index].page = g_counters.month.page;
+        history_list[history_index].visit = g_counters.month.visit;
+
+        g_settings.settings.history_index = history_index;
+    }
+
+    history_index = 0;
+    month_index = history_list[0].month;
+    year_index = history_list[0].year;
+    for (i = 0; i < MAXHISTLEN; i++) {
+        history_list[i].year = year_index;
+        history_list[i].month = month_index;
+        month_index++;
+        if (month_index > 12) {
+            month_index = 1;
+            year_index++;
+        }
+    }
+
+    if (g_settings.settings.verbosity >= VERBOSE3) {
+        VPRINT(VERBOSE3, "History Table Dump: --\n");
+        for (i = 0; i < MAXHISTLEN; i++) {
+            VPRINT(VERBOSE3, "%d %d %lu %lu %lu %.0f %d %d %lu %lu\n",
+                   history_list[i].month, history_list[i].year, history_list[i].hit, history_list[i].file, history_list[i].site, history_list[i].xfer, history_list[i].first_day,
+                   history_list[i].last_day, history_list[i].page, history_list[i].visit);
+        }
+        VPRINT(VERBOSE3, " --\n");
+    }
+}
+
+
 /*********************************************/
 /* SAVE_STATE - save internal data structs   */
 /*********************************************/
@@ -175,6 +285,7 @@ save_state()
     SNODEPTR sptr;
     INODEPTR iptr;
     ENODEPTR eptr;
+    SEGNODEPTR segptr;
 
     FILE *fp;
     int i;
@@ -205,12 +316,12 @@ save_state()
     /* Monthly totals for sites, urls, etc... */
     snprintf(buffer, sizeof(buffer), "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu %d %lu %lu\n", g_counters.month.hit, g_counters.month.file, g_counters.month.site,
              g_counters.month.url, g_counters.month.ref, g_counters.month.agent, g_counters.month.vol, g_counters.month.page, g_counters.month.visit, g_counters.month.user, 0,
-             g_counters.generic.bad, g_counters.generic.ignored);
+             g_counters.generic.bad_month, g_counters.generic.ignored_month);
     if (fputs(buffer, fp) == EOF)
         return 1;                               /* error exit */
 
     /* Daily totals for sites, urls, etc... */
-    snprintf(buffer, sizeof(buffer), "%u %lu %lu %d %d\n", 0, ht_hit, mh_hit, f_day, l_day);
+    snprintf(buffer, sizeof(buffer), "%u %lu %lu %d %d\n", 0, ht_hit, mh_hit, g_counters.month.first_day, g_counters.month.last_day);
     if (fputs(buffer, fp) == EOF)
         return 1;                               /* error exit */
 
@@ -368,14 +479,35 @@ save_state()
     if (fputs("# End Of Table - 404errors\n", fp) == EOF)
         return 1;                               /* error exit */
 
+    /* Save Segmenting Data. */
+    if (fputs("# -segmenting_referers- \n", fp) == EOF)
+        return 1;                               /* error exit */
+
+    for (i = 0; i < MAXHASH; i++) {
+        segptr = seg_ref_htab[i];
+        while (segptr != NULL) {
+            snprintf(buffer, sizeof(buffer), "%s\n%lu\n", segptr->string, segptr->tstamp);
+            if (fputs(buffer, fp) == EOF)
+                return 1;                       /* error exit */
+            segptr = segptr->next;
+        }
+    }
+    if (fputs("# End Of Table - segmenting_referers\n", fp) == EOF)
+        return 1;
+
     fclose(fp);                                 /* close data file...                            */
     return 0;                                   /* successful, return with good return code      */
 }
 
-/*********************************************/
-/* RESTORE_STATE - reload internal run data  */
-/*********************************************/
 
+/*********************************************
+ * RESTORE_STATE - reload internal run data
+ *
+ * Returns:
+ * 0 on successful load
+ * -1 on no file too load
+ * > 0 on any error in the load 
+ *********************************************/
 int
 restore_state()
 {
@@ -388,6 +520,7 @@ restore_state()
     struct snode t_snode;
     struct inode t_inode;
     struct enode t_enode;
+    struct seg_node t_segnode;
 
     char buffer[BUFSIZE];
     char buffer2[BUFSIZE], buffer3[BUFSIZE];
@@ -409,7 +542,7 @@ restore_state()
     if (fp == NULL) {
         /* Previous run data not found... */
         VPRINT(VERBOSE1, "%s\n", _("Previous run data not found..."));
-        return 0;                               /* return with ok code */
+        return -1;                              /* return with Not Found code */
     }
 
     /* Reading previous run data... */
@@ -474,14 +607,14 @@ restore_state()
         } else {
             sscanf(buffer, "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu %lu %lu %lu", &g_counters.month.hit, &g_counters.month.file, &g_counters.month.site, &g_counters.month.url,
                    &g_counters.month.ref, &g_counters.month.agent, &g_counters.month.vol, &g_counters.month.page, &g_counters.month.visit, &g_counters.month.user,
-                   &ignore_this_value, &g_counters.generic.bad, &g_counters.generic.ignored);
+                   &ignore_this_value, &g_counters.generic.bad_month, &g_counters.generic.ignored_month);
         }
     } else
         return 3;                               /* error exit */
 
     /* Daily totals for sites, urls, etc... */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-        sscanf(buffer, "%lu %lu %lu %d %d", &ignore_this_value, &ht_hit, &mh_hit, &f_day, &l_day);
+        sscanf(buffer, "%lu %lu %lu %d %d", &ignore_this_value, &ht_hit, &mh_hit, &g_counters.month.first_day, &g_counters.month.last_day);
     } else
         return 4;                               /* error exit */
 
@@ -569,7 +702,7 @@ restore_state()
         /* Good record, insert into hash table */
         if (put_unode(buffer2, t_unode.flag, t_unode.count, t_unode.xfer, &ul_bogus, t_unode.entry, t_unode.exit, t_unode.single_access, 0, um_htab)) {
             /* Error adding URL node, skipping ... */
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding URL node, skipping"), t_unode.string);
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding URL node, skipping"), buffer2);
         }
     }
 
@@ -621,7 +754,7 @@ restore_state()
             (buffer2, t_hnode.flag, t_hnode.count, t_hnode.files, t_hnode.xfer, &ul_bogus, t_hnode.visit, t_hnode.pages, t_hnode.tstamp, "", t_hnode.lasturl, sm_htab,
              t_hnode.lasturl_is_entry, false, &ignore_this_bool)) {
             /* Error adding host node (monthly), skipping .... */
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Host node (monthly), skipping"), t_hnode.string);
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Host node (monthly), skipping"), buffer2);
         }
     }
 
@@ -674,7 +807,7 @@ restore_state()
             (buffer2, t_hnode.flag, t_hnode.count, t_hnode.files, t_hnode.xfer, &ul_bogus, t_hnode.visit, t_hnode.pages, t_hnode.tstamp, "", t_hnode.lasturl, sd_htab,
              t_hnode.lasturl_is_entry, false, &ignore_this_bool)) {
             /* Error adding host node (daily), skipping .... */
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Host node (daily), skipping"), t_hnode.string);
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Host node (daily), skipping"), buffer2);
         }
     }
 
@@ -752,7 +885,7 @@ restore_state()
 
         /* insert node */
         if (put_snode(buffer2, t_snode.count, sr_htab)) {
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Search String Node, skipping"), t_snode.string);
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Search String Node, skipping"), buffer2);
         }
     }
 
@@ -780,7 +913,7 @@ restore_state()
         /* Good record, insert into hash table */
         if (put_inode(buffer2, t_inode.flag, t_inode.count, t_inode.files, t_inode.xfer, &ul_bogus, t_inode.visit, t_inode.tstamp, im_htab, false)) {
             /* Error adding username node, skipping .... */
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Username node, skipping"), t_inode.string);
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Username node, skipping"), buffer2);
         }
     }
 
@@ -810,15 +943,46 @@ restore_state()
             buffer3[strlen(buffer3) - 1] = '\0';        /* Chop off trailing newline */
 
             /* Good record, insert into hash table */
-            if (put_enode(buffer2, buffer3, OBJ_REG, t_enode.count, &g_counters.generic.error, ep_htab)) {
+            if (put_enode(buffer2, buffer3, OBJ_REG, t_enode.count, &g_counters.generic.error_month, ep_htab)) {
                 /* Error adding URL node, skipping ... */
-                ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding URL node, skipping"), t_unode.string);
+                ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding URL node, skipping"), buffer2);
+            }
+        }
+    }
+
+
+    /* Restore Segmenting Data. */
+    if (rundata_version >= 30901) {
+        if ((fgets(buffer, BUFSIZE, fp)) != NULL) {     /* Table header */
+            if (strncmp(buffer, "# -segmenting_referers- ", 10))
+                return 24;
+        } else
+            return 24;                          /* error exit */
+
+        while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
+            /* Check for end of table */
+            if (!strncmp(buffer2, "# End Of Table ", 15))
+                break;
+            buffer2[strlen(buffer2) - 1] = '\0';        /* Chop off trailing newline */
+
+            if ((fgets(buffer, BUFSIZE, fp)) == NULL)
+                return 25;                      /* error exit */
+            if (!isdigit((int) buffer[0]))
+                return 25;                      /* error exit */
+
+            /* load temporary node data */
+            sscanf(buffer, "%lu", &t_segnode.tstamp);
+
+            /* Good record, insert into hash table */
+            if (!put_segnode(buffer2, t_segnode.tstamp, seg_ref_htab)) {
+                /* Error adding segmenting by referrer node, skipping .... */
+                ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Segmenting By Referrer node, skipping"), buffer2);
             }
         }
     }
 
     fclose(fp);
-    check_dup = 1;                              /* enable duplicate checking */
+    g_settings.flags.is_first_run = false;      /* First run is false, as we're effectively mid-run */
     return 0;                                   /* return with ok code       */
 }
 
